;**************************** ISR CODE ****************************************
; Remark: The ISR code must always start at address 000. Time-critical
;         VP code (e.g. for A/D converters) should be placed before the
;         code of VPs that have a variable execution time, like this UART
;         code.
interrupt       ORG     $000
;
;**** Virtual Peripheral: Universal Asynchronous Receiver Transmitter (UART)
; This routine sends and receives serial RS232 data. It is configured for
; the frequently used "8-N-1" format (8 Bits, no parity, 1 stop bit).
; RECEIVE:  The rx_flag will be set as soon as a valid data byte is available.
;           It is then the responsibility of the code that processes the data
;           byte to reset the rx_flag again.
; TRANSMIT: The transmit routine expects the inverted data in the register-
;           pair tx_high, and tx_low, where the byte to be sent must be
;           inverted and stored in tx_high. In tx_low, Bit 7 must be set,
;           and the other bits are ignored. Then the number of bits to be sent
;           (10 = 1 start bit + 8 data bits + 1 stop bit) must be stored in
;           tx_count. As soon as tx_count contains a value > 0, the transmitter
;           starts sending the data. The calling application may test tx_count
;           in order to determine if a transmission is still in progress, i.e.
;           if tx_count > 0.
; This VP has variable execution times. Therefore, it should be located behind
; time-critical code (e.g. for A/D converters, timers, PWMs, etc.) in the ISR.
;
; Note: The transmit and receive code is independent from each other. If
;       the transmitter or the receiver is not required in an application,
;       its code may be deleted. Take care NOT to delete the initial 
;       bank serial instruction.
;
;       Input variables:  tx_low (bit 7 only)
;                         tx_high, tx_count
;
;       Output variables: rx_flag, rx_byte
;
;       Required clock cycles (in turbo mode):
;
;           Transmit:   9 cycles (while idle)
;                      19 cycles (while sending)
;                    +  1 cycle  for the common bank instruction
;
;           Receive:   9 cycles (while idle)
;                     16 cycles (at start)
;                     13 cycles (while reading the next bit)
;                     17 cycles (at the end of reception)
;
   bank    serial                  ;Select the "serial" bank            ;  1
:transmit
   clrb    tx_divide.baud_bit      ;Clear transmit timer flag           ;  2
   inc     tx_divide               ;Increment the counter               ;  3
   stz                             ;Set Z flag for next instruction     ;  4
   snb     tx_divide.baud_bit      ;Execute the transmit routine on     ;  5
                                   ; (2 ^ baud_bit)-th interrupt.
     test  tx_count                ;Are we sending data?                ;  6
   jz      :receive                ; no, continue with the receiver     ;  7
   clc                             ; yes, prepare stop bit, and         ;  8
   rr      tx_high                 ; shift to next bit                  ;  9
   rr      tx_low                                                       ; 10
   dec     tx_count                ;Decrement bit counter               ; 11
   movb    tx_pin, /tx_low.6       ;Output next bit                     ; 12

:receive
   movb    c, rx_pin               ;Store received bit in carry flag    ; 13
   test    rx_count                ;Are we receiving?                   ; 14
   jnz     :rxbit                  ; yes, continue receiving            ; 15
   mov     w, #9                   ; no, prepare 9 bits                 ; 16
   sc                              ; if no start bit, continue          ; 17
     mov   rx_count, w             ; if start bit, set bit counter      ; 18
   mov     rx_divide, #start_delay ;Set 1.5 bit periods                 ; 19

:rxbit
   djnz    rx_divide, :rxdone      ;In the "middle" of the next bit?    ; 20
   setb    rx_divide.baud_bit      ; yes, set 1 bit period              ; 21
   dec     rx_count                ;Last bit?                           ; 22
   sz                              ; no,                                ; 23
     rr    rx_byte                 ;  save bit                          ; 24
   snz                             ; yes,                               ; 25
     setb  rx_flag                 ;  set flag                          ; 26

:rxdone
     mov     w, #-int_period       ;Interrupt every int_period          ; 27
                                   ; clock cycles
                                   ; (163 for 19,200 Bd)
:end_int
     retiw                         ;Leave the ISR                       ; 28
